//+------------------------------------------------------------------+
//|                                Ferox Muti Timeframe Viewer v1.0  |
//+------------------------------------------------------------------+
#property copyright "Copyright 2020, Ferox Technologies Corp."
#property link "dat.ngtat@gmail.com"
#property version "1.0"
#property description "Ferox Muti Timeframe Viewer v1.0"
#property indicator_chart_window
#property strict

//---- extern parameters
extern ENUM_TIMEFRAMES view = PERIOD_D1; // View Period
extern int       bars_view=3;       // Count bars back
extern bool      show_tail=false;   // Show bars tail
extern color     UpColor=clrSteelBlue;       // Color bars up
extern color     DnColor=clrCoral;     // Color bars dn
extern color     TextColor=Black;   // Color bars tail text
extern int       vwidth = 2;        // Line width
extern bool      fill=false;        // Fill Color full bars

color vcolor;
string key;
double pip;

// up CornflowerBlue, SteelBlue; dn LightSalmon, IndianRed

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
{  key = DoubleToStr(rand()/1000,0)+"c";
   int digits = (int)MarketInfo(NULL,MODE_DIGITS);
   pip = MarketInfo(NULL,MODE_POINT)*(digits % 2 == 1 ? 10 : 1);
   return(INIT_SUCCEEDED);
}
//+------------------------------------------------------------------+
//| Custor indicator deinitialization function                       |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
{  DrawDelete(key);
}

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prevcalculated,
                const datetime& time[],
                const double& open[],
                const double& high[],
                const double& low[],
                const double& close[],
                const long& tick_volume[],
                const long& volume[],
                const int& spread[])
{  int i=0;
   datetime period=(datetime)Period();
   int limit = fmin(iBars(NULL,PERIOD_MN1),bars_view);
   if(bars_view==0) limit = iBars(NULL,PERIOD_MN1);
   if(view==Period()) return 0; // không vẽ

   while(i <= limit)
   {  double vopen=iOpen(NULL,view,i);
      double vclose=iClose(NULL,view,i);
      double vlow=iLow(NULL,view,i);
      double vhigh=iHigh(NULL,view,i);
      double iprice = vclose;
      if(vopen > vclose) iprice = vopen;

      datetime vtime = iTime(NULL,view,i);
      datetime vtime0= iTime(NULL,view,i-1)-period;
      datetime mtime = (vtime + vtime0)/2 + period; // lấy thời gian nữa chu kỳ view

      // Chỉnh lại nến view số 0 và nến 1
      if(i<=1)
      {  vtime = iTime(NULL,view,i);
         vtime0 = vtime + view*60 - period;
         mtime = (vtime + vtime0)/2 + period;
      }

      // vẽ thân nến
      string obj = key+(string)i+"bod";
      ObjectDelete(obj);
      ObjectCreate(obj,OBJ_RECTANGLE,0,0,vopen,0,vclose);
      vcolor=DnColor;
      if(vclose>=vopen) vcolor=UpColor;

      ObjectSet(obj,OBJPROP_TIME1,vtime);
      ObjectSet(obj,OBJPROP_TIME2,vtime0);
      ObjectSet(obj,OBJPROP_STYLE,STYLE_SOLID);
      ObjectSet(obj,OBJPROP_BACK,fill);
      ObjectSet(obj,OBJPROP_COLOR,vcolor);
      ObjectSet(obj,OBJPROP_WIDTH,vwidth);
      ObjectSet(obj,OBJPROP_RAY,false);
      ObjectSetText(obj,"close "+(string)vclose+" open "+(string)vopen,8,NULL,clrNONE);

      // vẽ đoạn đỉnh nến
      obj = key+(string)i+"top";
      ObjectDelete(obj);
      ObjectCreate(obj,OBJ_TREND,0,0,vhigh,0,iprice);

      ObjectSet(obj,OBJPROP_TIME1,mtime);
      ObjectSet(obj,OBJPROP_TIME2,mtime);
      ObjectSet(obj,OBJPROP_STYLE,STYLE_SOLID);
      ObjectSet(obj,OBJPROP_BACK,0);
      ObjectSet(obj,OBJPROP_COLOR,vcolor);
      ObjectSet(obj,OBJPROP_WIDTH,vwidth);
      ObjectSet(obj,OBJPROP_RAY,false);
      ObjectSetText(obj,"top "+(string)vhigh,8,NULL,clrNONE);

      // vẽ đoạn đuôi nến
      obj = key+(string)i+"bot";
      ObjectDelete(obj);
      iprice = vopen;
      if(vopen > vclose) iprice = vclose;
      ObjectCreate(obj,OBJ_TREND,0,0,vlow,0,iprice);

      ObjectSet(obj,OBJPROP_TIME1,mtime);
      ObjectSet(obj,OBJPROP_TIME2,mtime);
      ObjectSet(obj,OBJPROP_STYLE,STYLE_SOLID);
      ObjectSet(obj,OBJPROP_BACK,0);
      ObjectSet(obj,OBJPROP_COLOR,vcolor);
      ObjectSet(obj,OBJPROP_WIDTH,vwidth);
      ObjectSet(obj,OBJPROP_RAY,false);
      ObjectSetText(obj,"botom "+(string)vlow,8,NULL,clrNONE);

      // vẽ giá trị chiều cao nến view
      double range_bar = (vhigh-vlow)/pip;
      if(show_tail==true)
      {  ObjectDelete(key+(string)i+"txt");
         ObjectCreate(key+(string)i+"txt",OBJ_TEXT,0,mtime,vlow);
         ObjectSetText(key+(string)i+"txt",DoubleToStr(range_bar,0)+ " pips",8,NULL,TextColor);
      }

      //--- Biến trong while loop
      i++;
   }
   WindowRedraw();
   return 0;
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void DrawDelete(string find)
{  for(int i = ObjectsTotal() - 1; i >= 0;  i--)
   {  string x = ObjectName(i);
      if(StringSubstr(x,0,StringLen(find)) == find)
         ObjectDelete(x);
   }
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
